% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/newplots.R
\name{gf_ash}
\alias{gf_ash}
\alias{stat_ash}
\alias{geom_ash}
\title{Average Shifted Histograms}
\usage{
gf_ash(object = NULL, gformula = NULL, data = NULL, geom = "line",
  stat = "ash", position = "identity", show.legend = NA,
  show.help = NULL, inherit = TRUE, ...)

stat_ash(mapping = NULL, data = NULL, geom = "line",
  position = "identity", na.rm = FALSE, show.legend = NA,
  inherit.aes = TRUE, binwidth = NULL, adjust = 1, ...)

geom_ash(mapping = NULL, data = NULL, stat = "ash",
  position = "identity", na.rm = FALSE, show.legend = NA,
  inherit.aes = TRUE, binwidth = NULL, adjust = 1, ...)
}
\arguments{
\item{object}{When chaining, this holds an object produced in the earlier portions
of the chain.  Most users can safely ignore this argument.
See details and examples.}

\item{gformula}{A formula with shape \code{~x} or \code{y ~ x}.
\code{y} may be \code{..density..} or \code{..count..} or \code{..ndensity..} or \code{..ncount..}.
Faceting can be achieved by including \code{|} in the formula.}

\item{data}{A data frame with the variables to be plotted.}

\item{geom}{A character string naming the geom used to make the layer.}

\item{stat}{A character string naming the stat used to make the layer.}

\item{position}{Either a character string naming the position function used
for the layer or a position object returned from a call to a position function.}

\item{show.legend}{A logical indicating whether this layer should be included in
the legends.  \code{NA}, the default, includes layer in the legends if any
of the attributes of the layer are mapped.}

\item{show.help}{If \code{TRUE}, display some minimal help.}

\item{inherit}{A logical indicating whether default attributes are inherited.}

\item{...}{Additional arguments.  Typically these are
(a) ggplot2 aesthetics to be set with \code{attribute = value},
(b) ggplot2 aesthetics to be mapped with \code{attribute = ~expression}, or
(c) attributes of the layer as a whole, which are set with \code{attribute = value}.
Available attributes include
\code{alpha}, \code{color}, \code{fill}, \code{group}, \code{linetype}, \code{size}}

\item{mapping}{set of aesthetic mappings created by \code{\link[=aes]{aes()}}
or \code{\link[=aes_]{aes_()}}.}

\item{na.rm}{If FALSE (the default), removes missing values with a warning.
If TRUE silently removes missing values.}

\item{inherit.aes}{A logical indicating whether default aesthetics are inherited.}

\item{binwidth}{the width of the histogram bins.  If \code{NULL} (the default) the
binwidth will be chosen so that approximately 10 bins cover the data.  \code{adjust}
can be used to to increase or decrease \code{binwidth}.}

\item{adjust}{a numeric adjustment to \code{binwidth}.  Primarily useful when \code{binwidth} is
not specified.  Increasing \code{adjust} makes the plot smoother.}
}
\value{
a gg object
}
\description{
An ASH plot is the average over all histograms of a fixed bin width.
\code{geom_ash()} and \code{gf_ash()} provide ways to create ASH plots
using \pkg{ggplot2} or \pkg{ggformula}.
}
\examples{
gf_ash(~Sepal.Length, color = ~ Species, data = iris)
gf_ash(~Sepal.Length, color = ~ Species, data = iris, binwidth = 0.3)
gf_ash(~Sepal.Length, color = ~ Species, data = iris, adjust = 2)
ggplot(faithful, aes(x = eruptions)) +
  geom_histogram(aes(y = ..density..),
    fill = "lightskyblue", colour = "gray50", alpha = 0.2) +
  geom_ash(colour = "red") +
  geom_ash(colour = "forestgreen", adjust = 2) +
  geom_ash(colour = "navy", adjust = 1/2) +
  theme_minimal()
}
\seealso{
\code{\link[=geom_histogram]{geom_histogram()}}, \code{link{gf_histogram}()}.
}
